@@ -6,6 +6,7 @@ import sys
 from contextlib import contextmanager
 from select import select
 from logging import getLogger
+from hashlib import md5
 from weakref import WeakValueDictionary
 
 from .utils import (
@@ -14,7 +15,7 @@ from .utils import (
 
 
 __title__ = 'pq'
-__version__ = '1.8.1'
+__version__ = '1.9.0'
 __author__ = 'Malthe Borch'
 __license__ = 'BSD'
 
@@ -251,13 +252,19 @@ class Queue(object):
     def _conn(self):
         if self.pool:
             conn = self.pool.getconn()
-            yield conn
-            self.pool.putconn(conn)
+            try:
+                yield conn
+            finally:
+                self.pool.putconn(conn)
         else:
             yield self.conn
 
     def _listen(self, cursor):
-        cursor.execute('LISTEN "%s"', (Literal(self.name), ))
+        name = self.name
+        if len(name) > 63:
+            digest = md5(name.encode()).hexdigest()
+            name = "pq_" + digest
+        cursor.execute('LISTEN "%s"', (Literal(name), ))
 
     @prepared
     def _put_item(self, cursor):
@@ -298,7 +305,7 @@ class Queue(object):
                 WHERE
                   q_name = %(name)s AND
                   dequeued_at IS NULL
-                ORDER BY schedule_at nulls first, expected_at nulls last
+                ORDER BY schedule_at nulls first, expected_at nulls last, id
                 FOR UPDATE SKIP LOCKED
                 LIMIT 1
               ),
