@@ -1,6 +1,4 @@
-do $$ begin
-
-CREATE TABLE %(name)s (
+create table if not exists %(name)s (
   id          bigserial    PRIMARY KEY,
   enqueued_at timestamptz  NOT NULL DEFAULT current_timestamp,
   dequeued_at timestamptz,
@@ -10,22 +8,26 @@ CREATE TABLE %(name)s (
   data        json         NOT NULL
 );
 
-end $$ language plpgsql;
-
-create index priority_idx_%(name)s on %(name)s
-    (schedule_at nulls first, expected_at nulls first, q_name)
+create index if not exists priority_idx_%(name)s on %(name)s
+      (schedule_at nulls first, expected_at nulls last, q_name)
     where dequeued_at is null
           and q_name = '%(name)s';
 
-create index priority_idx_no_%(name)s on %(name)s
-    (schedule_at nulls first, expected_at nulls first, q_name)
+create index if not exists priority_idx_no_%(name)s on %(name)s
+    (schedule_at nulls first, expected_at nulls last, q_name)
     where dequeued_at is null
           and q_name != '%(name)s';
 
 drop function if exists pq_notify() cascade;
 
 create function pq_notify() returns trigger as $$ begin
-  perform pg_notify(new.q_name, '');
+  perform pg_notify(
+    case length(new.q_name) > 63
+      when true then 'pq_' || md5(new.q_name)
+      else new.q_name
+    end,
+    ''
+  );
   return null;
 end $$ language plpgsql;
 
